/*
 * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.nodes.function.builtins;

import com.oracle.graal.python.nodes.PGuards;
import com.oracle.graal.python.nodes.function.PythonBuiltinBaseNode;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.nodes.UnexpectedResultException;

public abstract class PythonUnaryBuiltinNode extends PythonBuiltinBaseNode {

    public Object call(VirtualFrame frame, Object arg) {
        return execute(frame, arg);
    }

    public boolean callBool(VirtualFrame VirtualFrame, boolean arg) throws UnexpectedResultException {
        return executeBool(VirtualFrame, arg);
    }

    public int callInt(VirtualFrame VirtualFrame, int arg) throws UnexpectedResultException {
        return executeInt(VirtualFrame, arg);
    }

    public long callLong(VirtualFrame VirtualFrame, long arg) throws UnexpectedResultException {
        return executeLong(VirtualFrame, arg);
    }

    public double callDouble(VirtualFrame VirtualFrame, double arg) throws UnexpectedResultException {
        return executeDouble(VirtualFrame, arg);
    }

    public boolean callBool(VirtualFrame VirtualFrame, int arg) throws UnexpectedResultException {
        return executeBool(VirtualFrame, arg);
    }

    public boolean callBool(VirtualFrame VirtualFrame, long arg) throws UnexpectedResultException {
        return executeBool(VirtualFrame, arg);
    }

    public boolean callBool(VirtualFrame VirtualFrame, double arg) throws UnexpectedResultException {
        return executeBool(VirtualFrame, arg);
    }

    // ------------------
    // execute methods:

    protected abstract Object execute(VirtualFrame VirtualFrame, Object arg);

    protected boolean executeBool(VirtualFrame VirtualFrame, boolean arg) throws UnexpectedResultException {
        return PGuards.expectBoolean(execute(VirtualFrame, arg));
    }

    protected int executeInt(VirtualFrame VirtualFrame, int arg) throws UnexpectedResultException {
        return PGuards.expectInteger(execute(VirtualFrame, arg));
    }

    protected long executeLong(VirtualFrame VirtualFrame, long arg) throws UnexpectedResultException {
        return PGuards.expectLong(execute(VirtualFrame, arg));
    }

    protected double executeDouble(VirtualFrame VirtualFrame, double arg) throws UnexpectedResultException {
        return PGuards.expectDouble(execute(VirtualFrame, arg));
    }

    protected boolean executeBool(VirtualFrame VirtualFrame, int arg) throws UnexpectedResultException {
        return PGuards.expectBoolean(execute(VirtualFrame, arg));
    }

    protected boolean executeBool(VirtualFrame VirtualFrame, long arg) throws UnexpectedResultException {
        return PGuards.expectBoolean(execute(VirtualFrame, arg));
    }

    protected boolean executeBool(VirtualFrame VirtualFrame, double arg) throws UnexpectedResultException {
        return PGuards.expectBoolean(execute(VirtualFrame, arg));
    }
}
